const IDTO = global.IDTO || require('./IDTO')
const AssetDTO = global.AssetDTO || require('./AssetDTO')
global.JsmexSymbol = global.JsmexSymbol || require('../utils/Symbol')

/**
 * MarketDTO class
 *
 * @author jsb4ch <jsb4ch@jsmex.io>
 * @since 1.0.0
*/
class MarketDTO extends IDTO {
  constructor (uid) {
    super(uid)

    this.exchangeName = ''
    this.stock = {}
    this.money = {}
    this.symbolBuild()
  }

  /**
   * MarketDTO print ticker with specific format
   *
   * @author jsb4ch <jsb4ch@jsmex.io>
   * @since 1.0.0
  */
  format ({ prefix = '', separator = '', iso = false } = {}) {
    return iso ? `${prefix}${this.stock.iso}${separator}${this.money.iso}` : `${prefix}${this.stock.alias}${separator}${this.money.alias}`
  }

  /**
   * MarketDTO symbolId gen method
   *
   * @author jsb4ch <jsb4ch@jsmex.io>
   * @since 1.0.86
  */
  symbolBuild () {
    this.symbolId = `@${this.exchangeName}/${this.stock.iso || this.stock.alias}-${this.money.iso || this.money.alias}`
  }

  /**
   * MarketDTO validation method
   *
   * @author jsb4ch <jsb4ch@jsmex.io>
   * @since 1.0.0
  */
  validate () {
    this.symbolBuild()
    if (typeof this.exchangeName !== 'string') {
      throw new Error('MarketDTO exchangeName is not a string')
    }
    if (!(AssetDTO.isValid(this.stock))) {
      let error = { code: 1001, isCritical: true, message: 'stock is not instance of asset dto' }
      throw error
    }
    if (!(AssetDTO.isValid(this.money))) {
      let error = { code: 1002, isCritical: true, message: 'money is not instance of asset dto' }
      throw error
    }
  }

  /**
   * MarketDTO toObject method
   *
   * @author jsb4ch <jsb4ch@jsmex.io>
   * @since 1.0.33
   * @returns {Object}
  */
  toObject () {
    return {
      uid: this.uid,
      symbolId: this.symbolId,
      exchangeName: this.exchangeName,
      stock: this.stock.toObject(),
      money: this.money.toObject()
    }
  }

  /**
   * MarketDTO static from default object method
   *
   * @author jsb4ch <jsb4ch@jsmex.io>
   * @since 1.0.0
  */
  static fromObject (data) {
    let marketDTO = new MarketDTO(data.uid)
    marketDTO.exchangeName = data.exchangeName
    marketDTO.money = AssetDTO.fromObject(data.money)
    marketDTO.stock = AssetDTO.fromObject(data.stock)
    marketDTO.validate()
    return marketDTO
  }

  /**
   * MarketDTO formatFromSymbol method
   *
   * @author jsb4ch <jsb4ch@jsmex.io>
   * @since 1.0.50
  */
  static formatFromSymbol (...args) {
    return global.JsmexSymbol.formatFromSymbol(...args)
  }

  /**
   * MarketDTO exchangeNameBySymbol method
   *
   * @author jsb4ch <jsb4ch@jsmex.io>
   * @since 1.0.70
   * @param {String} symbol
   * @return {String}
  */
  static exchangeNameBySymbol (symbol) {
    return global.JsmexSymbol.exchangeName(symbol)
  }

  /**
   * MarketDTO isSymbol method
   *
   * @author jsb4ch <jsb4ch@jsmex.io>
   * @since 1.0.73
   * @param {String} symbol
   * @return {Boolean}
  */
  static isSymbol (symbol) {
    return global.JsmexSymbol.isSymbol(symbol)
  }
}

module.exports = MarketDTO
